@@ -14,6 +14,7 @@ use crate::generate_env_vars_reference::Args as GenerateEnvVarsReferenceArgs;
 use crate::generate_json_schema::Args as GenerateJsonSchemaArgs;
 use crate::generate_options_reference::Args as GenerateOptionsReferenceArgs;
 use crate::generate_sysconfig_mappings::Args as GenerateSysconfigMetadataArgs;
+use crate::list_packages::ListPackagesArgs;
 #[cfg(feature = "render")]
 use crate::render_benchmarks::RenderBenchmarksArgs;
 use crate::validate_zip::ValidateZipArgs;
@@ -27,6 +28,7 @@ mod generate_env_vars_reference;
 mod generate_json_schema;
 mod generate_options_reference;
 mod generate_sysconfig_mappings;
+mod list_packages;
 mod render_benchmarks;
 mod validate_zip;
 mod wheel_metadata;
@@ -43,6 +45,8 @@ enum Cli {
     Compile(CompileArgs),
     /// Remove all `.pyc` in the tree.
     ClearCompile(ClearCompileArgs),
+    /// List all packages from a Simple API index.
+    ListPackages(ListPackagesArgs),
     /// Run all code and documentation generation steps.
     GenerateAll(GenerateAllArgs),
     /// Generate JSON schema for the TOML configuration file.
@@ -69,6 +73,7 @@ pub async fn run() -> Result<()> {
         Cli::ValidateZip(args) => validate_zip::validate_zip(args, environment).await?,
         Cli::Compile(args) => compile::compile(args).await?,
         Cli::ClearCompile(args) => clear_compile::clear_compile(&args)?,
+        Cli::ListPackages(args) => list_packages::list_packages(args, environment).await?,
         Cli::GenerateAll(args) => generate_all::main(&args).await?,
         Cli::GenerateJSONSchema(args) => generate_json_schema::main(&args)?,
         Cli::GenerateOptionsReference(args) => generate_options_reference::main(&args)?,
