@@ -336,6 +336,7 @@ fn warn_uv_toml_masked_fields(options: &Options) {
                 no_build_package,
                 no_binary,
                 no_binary_package,
+                torch_backend,
             },
         install_mirrors:
             PythonInstallMirrors {
@@ -497,6 +498,9 @@ fn warn_uv_toml_masked_fields(options: &Options) {
     if no_binary_package.is_some() {
         masked_fields.push("no-binary-package");
     }
+    if torch_backend.is_some() {
+        masked_fields.push("torch-backend");
+    }
     if python_install_mirror.is_some() {
         masked_fields.push("python-install-mirror");
     }
@@ -590,6 +594,7 @@ pub struct EnvironmentOptions {
     pub python_install_registry: Option<bool>,
     pub install_mirrors: PythonInstallMirrors,
     pub log_context: Option<bool>,
+    pub lfs: Option<bool>,
     pub http_timeout: Duration,
     pub http_retries: u32,
     pub upload_http_timeout: Duration,
@@ -636,6 +641,7 @@ impl EnvironmentOptions {
                 )?,
             },
             log_context: parse_boolish_environment_variable(EnvVars::UV_LOG_CONTEXT)?,
+            lfs: parse_boolish_environment_variable(EnvVars::UV_GIT_LFS)?,
             upload_http_timeout: parse_integer_environment_variable(
                 EnvVars::UV_UPLOAD_HTTP_TIMEOUT,
             )?
