@@ -310,7 +310,12 @@ impl TorchStrategy {
             TorchSource::PyTorch => {
                 matches!(
                     package_name.as_str(),
-                    "torch"
+                    "pytorch-triton"
+                        | "pytorch-triton-rocm"
+                        | "pytorch-triton-xpu"
+                        | "torch"
+                        | "torch-tensorrt"
+                        | "torchao"
                         | "torcharrow"
                         | "torchaudio"
                         | "torchcsprng"
@@ -320,9 +325,6 @@ impl TorchStrategy {
                         | "torchtext"
                         | "torchvision"
                         | "triton"
-                        | "pytorch-triton"
-                        | "pytorch-triton-rocm"
-                        | "pytorch-triton-xpu"
                 )
             }
             TorchSource::Pyx => {
@@ -334,12 +336,16 @@ impl TorchStrategy {
                         | "megablocks"
                         | "natten"
                         | "pyg-lib"
+                        | "pytorch-triton"
+                        | "pytorch-triton-rocm"
+                        | "pytorch-triton-xpu"
+                        | "torch"
                         | "torch-cluster"
                         | "torch-scatter"
                         | "torch-sparse"
                         | "torch-spline-conv"
-                        | "vllm"
-                        | "torch"
+                        | "torch-tensorrt"
+                        | "torchao"
                         | "torcharrow"
                         | "torchaudio"
                         | "torchcsprng"
@@ -349,9 +355,7 @@ impl TorchStrategy {
                         | "torchtext"
                         | "torchvision"
                         | "triton"
-                        | "pytorch-triton"
-                        | "pytorch-triton-rocm"
-                        | "pytorch-triton-xpu"
+                        | "vllm"
                 )
             }
         }
@@ -365,13 +369,14 @@ impl TorchStrategy {
     pub fn has_system_dependency(&self, package_name: &PackageName) -> bool {
         matches!(
             package_name.as_str(),
-            "flash-attn"
+            "deepspeed"
+                | "flash-attn"
                 | "flash-attn-3"
                 | "megablocks"
                 | "natten"
-                | "deepspeed"
-                | "vllm"
                 | "torch"
+                | "torch-tensorrt"
+                | "torchao"
                 | "torcharrow"
                 | "torchaudio"
                 | "torchcsprng"
@@ -379,6 +384,7 @@ impl TorchStrategy {
                 | "torchdistx"
                 | "torchtext"
                 | "torchvision"
+                | "vllm"
         )
     }
 
