@@ -1,6 +1,7 @@
 use std::fmt::Write;
 
 use anyhow::Result;
+use diskus::DiskUsage;
 
 use crate::commands::{ExitStatus, human_readable_bytes};
 use crate::printer::Printer;
@@ -31,16 +32,9 @@ pub(crate) fn cache_size(
         return Ok(ExitStatus::Success);
     }
 
-    // Walk the entire cache root
-    let total_bytes: u64 = walkdir::WalkDir::new(cache.root())
-        .follow_links(false)
-        .into_iter()
-        .filter_map(Result::ok)
-        .filter_map(|entry| match entry.metadata() {
-            Ok(metadata) if metadata.is_file() => Some(metadata.len()),
-            _ => None,
-        })
-        .sum();
+    let disk_usage = DiskUsage::new(vec![cache.root().to_path_buf()]);
+
+    let total_bytes = disk_usage.count_ignoring_errors();
 
     if human_readable {
         let (bytes, unit) = human_readable_bytes(total_bytes);
