@@ -19,15 +19,17 @@ use uv_configuration::{
 use uv_dispatch::BuildDispatch;
 use uv_distribution::{DistributionDatabase, SourcedDependencyGroups};
 use uv_distribution_types::{
-    CachedDist, Diagnostic, InstalledDist, LocalDist, NameRequirementSpecification, Requirement,
-    ResolutionDiagnostic, UnresolvedRequirement, UnresolvedRequirementSpecification,
+    CachedDist, Diagnostic, Dist, InstalledDist, InstalledVersion, LocalDist,
+    NameRequirementSpecification, Requirement, ResolutionDiagnostic, UnresolvedRequirement,
+    UnresolvedRequirementSpecification, VersionOrUrlRef,
 };
 use uv_distribution_types::{DistributionMetadata, InstalledMetadata, Name, Resolution};
 use uv_fs::Simplified;
 use uv_install_wheel::LinkMode;
 use uv_installer::{InstallationStrategy, Plan, Planner, Preparer, SitePackages};
 use uv_normalize::PackageName;
-use uv_pep508::{MarkerEnvironment, RequirementOrigin};
+use uv_pep440::Version;
+use uv_pep508::{MarkerEnvironment, RequirementOrigin, VerbatimUrl};
 use uv_platform_tags::Tags;
 use uv_preview::Preview;
 use uv_pypi_types::{Conflicts, ResolverMarkerEnvironment};
@@ -44,9 +46,9 @@ use uv_tool::InstalledTools;
 use uv_types::{BuildContext, HashStrategy, InFlight, InstalledPackagesProvider};
 use uv_warnings::warn_user;
 
-use crate::commands::pip::loggers::{DefaultInstallLogger, InstallLogger, ResolveLogger};
+use crate::commands::compile_bytecode;
+use crate::commands::pip::loggers::{InstallLogger, ResolveLogger};
 use crate::commands::reporters::{InstallReporter, PrepareReporter, ResolverReporter};
-use crate::commands::{ChangeEventKind, DryRunEvent, compile_bytecode};
 use crate::printer::Printer;
 
 /// Consolidate the requirements for an installation.
@@ -381,30 +383,104 @@ pub(crate) enum Modifications {
     Exact,
 }
 
+/// A distribution which was or would be modified
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[allow(clippy::large_enum_variant)]
+pub(crate) enum ChangedDist {
+    Local(LocalDist),
+    Remote(Arc<Dist>),
+}
+
+impl Name for ChangedDist {
+    fn name(&self) -> &PackageName {
+        match self {
+            Self::Local(dist) => dist.name(),
+            Self::Remote(dist) => dist.name(),
+        }
+    }
+}
+
+/// The [`Version`] or [`VerbatimUrl`] for a changed dist.
+#[derive(Debug, PartialOrd, Ord, PartialEq, Eq, Hash)]
+pub(crate) enum ShortSpecifier<'a> {
+    Version(&'a Version),
+    Url(&'a VerbatimUrl),
+}
+
+impl std::fmt::Display for ShortSpecifier<'_> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::Version(version) => version.fmt(f),
+            Self::Url(url) => write!(f, " @ {url}"),
+        }
+    }
+}
+
+/// The [`InstalledVersion`] or [`VerbatimUrl`] for a changed dist.
+#[derive(Debug, PartialOrd, Ord, PartialEq, Eq, Hash)]
+pub(crate) enum LongSpecifier<'a> {
+    InstalledVersion(InstalledVersion<'a>),
+    Url(&'a VerbatimUrl),
+}
+
+impl std::fmt::Display for LongSpecifier<'_> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::InstalledVersion(version) => version.fmt(f),
+            Self::Url(url) => write!(f, " @ {url}"),
+        }
+    }
+}
+
+impl ChangedDist {
+    pub(crate) fn short_specifier(&self) -> ShortSpecifier<'_> {
+        match self {
+            Self::Local(dist) => ShortSpecifier::Version(dist.installed_version().version()),
+            Self::Remote(dist) => match dist.version_or_url() {
+                VersionOrUrlRef::Version(version) => ShortSpecifier::Version(version),
+                VersionOrUrlRef::Url(url) => ShortSpecifier::Url(url),
+            },
+        }
+    }
+
+    pub(crate) fn long_specifier(&self) -> LongSpecifier<'_> {
+        match self {
+            Self::Local(dist) => LongSpecifier::InstalledVersion(dist.installed_version()),
+            Self::Remote(dist) => match dist.version_or_url() {
+                VersionOrUrlRef::Version(version) => {
+                    LongSpecifier::InstalledVersion(InstalledVersion::Version(version))
+                }
+                VersionOrUrlRef::Url(url) => LongSpecifier::Url(url),
+            },
+        }
+    }
+}
+
 /// A summary of the changes made to the environment during an installation.
 #[derive(Debug, Clone, Default)]
 pub(crate) struct Changelog {
     /// The distributions that were installed.
-    pub(crate) installed: HashSet<LocalDist>,
+    pub(crate) installed: HashSet<ChangedDist>,
     /// The distributions that were uninstalled.
-    pub(crate) uninstalled: HashSet<LocalDist>,
+    pub(crate) uninstalled: HashSet<ChangedDist>,
     /// The distributions that were reinstalled.
-    pub(crate) reinstalled: HashSet<LocalDist>,
+    pub(crate) reinstalled: HashSet<ChangedDist>,
 }
 
 impl Changelog {
-    /// Create a [`Changelog`] from a list of installed and uninstalled distributions.
-    pub(crate) fn new(installed: Vec<CachedDist>, uninstalled: Vec<InstalledDist>) -> Self {
+    /// Create a [`Changelog`] from two iterators of [`ChangedDist`]s.
+    pub(crate) fn new<I, U>(installed: I, uninstalled: U) -> Self
+    where
+        I: IntoIterator<Item = ChangedDist>,
+        U: IntoIterator<Item = ChangedDist>,
+    {
         // SAFETY: This is allowed because `LocalDist` implements `Hash` and `Eq` based solely on
         // the inner `kind`, and omits the types that rely on internal mutability.
         #[allow(clippy::mutable_key_type)]
-        let mut uninstalled: HashSet<_> = uninstalled.into_iter().map(LocalDist::from).collect();
-
+        let mut uninstalled: HashSet<_> = uninstalled.into_iter().collect();
         let (reinstalled, installed): (HashSet<_>, HashSet<_>) = installed
             .into_iter()
-            .map(LocalDist::from)
             .partition(|dist| uninstalled.contains(dist));
-
         uninstalled.retain(|dist| !reinstalled.contains(dist));
 
         Self {
@@ -414,13 +490,21 @@ impl Changelog {
         }
     }
 
+    /// Create a [`Changelog`] from a list of local distributions.
+    pub(crate) fn from_local(installed: Vec<CachedDist>, uninstalled: Vec<InstalledDist>) -> Self {
+        Self::new(
+            installed
+                .into_iter()
+                .map(|dist| ChangedDist::Local(dist.into())),
+            uninstalled
+                .into_iter()
+                .map(|dist| ChangedDist::Local(dist.into())),
+        )
+    }
+
     /// Create a [`Changelog`] from a list of installed distributions.
     pub(crate) fn from_installed(installed: Vec<CachedDist>) -> Self {
-        Self {
-            installed: installed.into_iter().map(LocalDist::from).collect(),
-            uninstalled: HashSet::default(),
-            reinstalled: HashSet::default(),
-        }
+        Self::from_local(installed, Vec::new())
     }
 
     /// Returns `true` if the changelog includes a distribution with the given name, either via
@@ -485,8 +569,15 @@ pub(crate) async fn install(
         .context("Failed to determine installation plan")?;
 
     if dry_run.enabled() {
-        report_dry_run(dry_run, resolution, plan, modifications, start, printer)?;
-        return Ok(Changelog::default());
+        return report_dry_run(
+            dry_run,
+            resolution,
+            plan,
+            modifications,
+            start,
+            logger.as_ref(),
+            printer,
+        );
     }
 
     let Plan {
@@ -509,7 +600,7 @@ pub(crate) async fn install(
         && extraneous.is_empty()
         && !compile
     {
-        logger.on_audit(resolution.len(), start, printer)?;
+        logger.on_audit(resolution.len(), start, printer, dry_run)?;
         return Ok(Changelog::default());
     }
 
@@ -590,10 +681,10 @@ pub(crate) async fn install(
     }
 
     // Construct a summary of the changes made to the environment.
-    let changelog = Changelog::new(installs, uninstalls);
+    let changelog = Changelog::from_local(installs, uninstalls);
 
     // Notify the user of any environment modifications.
-    logger.on_complete(&changelog, printer)?;
+    logger.on_complete(&changelog, printer, dry_run)?;
 
     Ok(changelog)
 }
@@ -660,7 +751,13 @@ async fn execute_plan(
             .prepare(remote.clone(), in_flight, resolution)
             .await?;
 
-        logger.on_prepare(wheels.len(), phase.map(InstallPhase::label), start, printer)?;
+        logger.on_prepare(
+            wheels.len(),
+            phase.map(InstallPhase::label),
+            start,
+            printer,
+            DryRun::Disabled,
+        )?;
 
         wheels
     };
@@ -702,7 +799,7 @@ async fn execute_plan(
             }
         }
 
-        logger.on_uninstall(uninstalls.len(), start, printer)?;
+        logger.on_uninstall(uninstalls.len(), start, printer, DryRun::Disabled)?;
     }
 
     // Install the resolved distributions.
@@ -721,7 +818,7 @@ async fn execute_plan(
             // task.
             .install_blocking(installs)?;
 
-        logger.on_install(installs.len(), start, printer)?;
+        logger.on_install(installs.len(), start, printer, DryRun::Disabled)?;
     }
 
     Ok((installs, uninstalls))
@@ -840,8 +937,9 @@ fn report_dry_run(
     plan: Plan,
     modifications: Modifications,
     start: std::time::Instant,
+    logger: &dyn InstallLogger,
     printer: Printer,
-) -> Result<(), Error> {
+) -> Result<Changelog, Error> {
     let Plan {
         cached,
         remote,
@@ -857,25 +955,15 @@ fn report_dry_run(
 
     // Nothing to do.
     if remote.is_empty() && cached.is_empty() && reinstalls.is_empty() && extraneous.is_empty() {
-        DefaultInstallLogger.on_audit(resolution.len(), start, printer)?;
-        writeln!(printer.stderr(), "Would make no changes")?;
-        return Ok(());
+        logger.on_audit(resolution.len(), start, printer, dry_run)?;
+        return Ok(Changelog::default());
     }
 
     // Download, build, and unzip any missing distributions.
     let wheels = if remote.is_empty() {
         vec![]
     } else {
-        let s = if remote.len() == 1 { "" } else { "s" };
-        writeln!(
-            printer.stderr(),
-            "{}",
-            format!(
-                "Would download {}",
-                format!("{} package{}", remote.len(), s).bold(),
-            )
-            .dimmed()
-        )?;
+        logger.on_prepare(remote.len(), None, start, printer, dry_run)?;
         remote.clone()
     };
 
@@ -883,87 +971,35 @@ fn report_dry_run(
     let uninstalls = extraneous.len() + reinstalls.len();
 
     if uninstalls > 0 {
-        let s = if uninstalls == 1 { "" } else { "s" };
-        writeln!(
-            printer.stderr(),
-            "{}",
-            format!(
-                "Would uninstall {}",
-                format!("{uninstalls} package{s}").bold(),
-            )
-            .dimmed()
-        )?;
+        logger.on_uninstall(uninstalls, start, printer, dry_run)?;
     }
 
     // Install the resolved distributions.
     let installs = wheels.len() + cached.len();
 
     if installs > 0 {
-        let s = if installs == 1 { "" } else { "s" };
-        writeln!(
-            printer.stderr(),
-            "{}",
-            format!("Would install {}", format!("{installs} package{s}").bold()).dimmed()
-        )?;
+        logger.on_install(installs, start, printer, dry_run)?;
     }
 
-    // TODO(charlie): DRY this up with `report_modifications`. The types don't quite line up.
-    for event in reinstalls
+    let uninstalled = reinstalls
         .into_iter()
-        .chain(extraneous.into_iter())
-        .map(|distribution| DryRunEvent {
-            name: distribution.name().clone(),
-            version: distribution.installed_version().to_string(),
-            kind: ChangeEventKind::Removed,
-        })
-        .chain(wheels.into_iter().map(|distribution| DryRunEvent {
-            name: distribution.name().clone(),
-            version: distribution.version_or_url().to_string(),
-            kind: ChangeEventKind::Added,
-        }))
-        .chain(cached.into_iter().map(|distribution| DryRunEvent {
-            name: distribution.name().clone(),
-            version: distribution.installed_version().to_string(),
-            kind: ChangeEventKind::Added,
-        }))
-        .sorted_unstable_by(|a, b| a.name.cmp(&b.name).then_with(|| a.kind.cmp(&b.kind)))
-    {
-        match event.kind {
-            ChangeEventKind::Added => {
-                writeln!(
-                    printer.stderr(),
-                    " {} {}{}",
-                    "+".green(),
-                    event.name.bold(),
-                    event.version.dimmed()
-                )?;
-            }
-            ChangeEventKind::Removed => {
-                writeln!(
-                    printer.stderr(),
-                    " {} {}{}",
-                    "-".red(),
-                    event.name.bold(),
-                    event.version.dimmed()
-                )?;
-            }
-            ChangeEventKind::Reinstalled => {
-                writeln!(
-                    printer.stderr(),
-                    " {} {}{}",
-                    "~".yellow(),
-                    event.name.bold(),
-                    event.version.dimmed()
-                )?;
-            }
-        }
-    }
+        .chain(extraneous)
+        .map(|dist| ChangedDist::Local(dist.into()));
+    let installed = wheels.into_iter().map(ChangedDist::Remote).chain(
+        cached
+            .into_iter()
+            .map(|dist| ChangedDist::Local(dist.into())),
+    );
+
+    let changelog = Changelog::new(installed, uninstalled);
+
+    logger.on_complete(&changelog, printer, dry_run)?;
 
     if matches!(dry_run, DryRun::Check) {
         return Err(Error::OutdatedEnvironment);
     }
 
-    Ok(())
+    Ok(changelog)
 }
 
 /// Report any diagnostics on resolved distributions.
