@@ -68,7 +68,7 @@ fn help() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command [env: UV_WORKING_DIR=]
           --project <PROJECT>
-              Run the command within the given project directory [env: UV_PROJECT=]
+              Discover a project in the given directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -149,7 +149,7 @@ fn help_flag() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command [env: UV_WORKING_DIR=]
           --project <PROJECT>
-              Run the command within the given project directory [env: UV_PROJECT=]
+              Discover a project in the given directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -229,7 +229,7 @@ fn help_short_flag() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command [env: UV_WORKING_DIR=]
           --project <PROJECT>
-              Run the command within the given project directory [env: UV_PROJECT=]
+              Discover a project in the given directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -419,7 +419,7 @@ fn help_subcommand() {
               [env: UV_WORKING_DIR=]
 
           --project <PROJECT>
-              Run the command within the given project directory.
+              Discover a project in the given directory.
               
               All `pyproject.toml`, `uv.toml`, and `.python-version` files will be discovered by walking
               up the directory tree from the project root, as will the project's virtual environment
@@ -692,7 +692,7 @@ fn help_subsubcommand() {
               [env: UV_WORKING_DIR=]
 
           --project <PROJECT>
-              Run the command within the given project directory.
+              Discover a project in the given directory.
               
               All `pyproject.toml`, `uv.toml`, and `.python-version` files will be discovered by walking
               up the directory tree from the project root, as will the project's virtual environment
@@ -783,7 +783,7 @@ fn help_flag_subcommand() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command [env: UV_WORKING_DIR=]
           --project <PROJECT>
-              Run the command within the given project directory [env: UV_PROJECT=]
+              Discover a project in the given directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -864,7 +864,7 @@ fn help_flag_subsubcommand() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command [env: UV_WORKING_DIR=]
           --project <PROJECT>
-              Run the command within the given project directory [env: UV_PROJECT=]
+              Discover a project in the given directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -1026,7 +1026,7 @@ fn help_with_global_option() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command [env: UV_WORKING_DIR=]
           --project <PROJECT>
-              Run the command within the given project directory [env: UV_PROJECT=]
+              Discover a project in the given directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
@@ -1149,7 +1149,7 @@ fn help_with_no_pager() {
           --directory <DIRECTORY>
               Change to the given directory prior to running the command [env: UV_WORKING_DIR=]
           --project <PROJECT>
-              Run the command within the given project directory [env: UV_PROJECT=]
+              Discover a project in the given directory [env: UV_PROJECT=]
           --config-file <CONFIG_FILE>
               The path to a `uv.toml` file to use for configuration [env: UV_CONFIG_FILE=]
           --no-config
