@@ -22,3 +22,6 @@ uv run "$project_root/scripts/generate-crate-readmes.py"
 echo "Updating lockfile..."
 cargo update -p uv
 pushd crates/uv-trampoline; cargo update -p uv-trampoline; popd
+
+echo "Generating JSON schema..."
+cargo dev generate-json-schema
